import { EventEmitter, Directive, ElementRef, Output, HostListener, Input, Renderer2, NgModule } from '@angular/core';
import { NG_VALIDATORS, FormsModule } from '@angular/forms';

class VonFormValidateDirective {
    constructor(el) {
        this.el = el;
        this.customSubmit = new EventEmitter();
        this.onSubmitEvent = (e) => {
            e.preventDefault();
            const elements = this.el.nativeElement.querySelectorAll('[validation]');
            this.isValid = true;
            this.isFocused = false;
            elements.forEach((el) => {
                el.dispatchEvent(new Event('executeValidation'));
                const elValid = el.getAttribute('validation') === 'true';
                if (this.isValid) {
                    this.isValid = elValid;
                }
                if (!elValid && !this.isFocused) {
                    this.isFocused = true;
                    el.focus();
                }
            });
            if (this.isValid) {
                this.customSubmit.emit(e);
            }
        };
    }
    ngOnInit() { }
}
VonFormValidateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[validate]'
            },] }
];
VonFormValidateDirective.ctorParameters = () => [
    { type: ElementRef }
];
VonFormValidateDirective.propDecorators = {
    customSubmit: [{ type: Output, args: ['validate',] }],
    onSubmitEvent: [{ type: HostListener, args: ['submit', ['$event'],] }]
};

class VonFormValidationBase {
    constructor(element) {
        this.element = element;
        this.validator = {};
        this.message = '';
        this.messages = {
            requiredMessage: '',
            equalToMessage: ''
        };
        this.getCustomValidators = (value) => {
            return {};
        };
        this.getClosestForm = (element) => {
            if (element.tagName === 'FORM' || element.tagName === 'BODY') {
                return element;
            }
            const parent = element.parentElement;
            if (parent) {
                return this.getClosestForm(parent);
            }
            return element;
        };
        this.getLabelText = () => {
            let labelText = this.name || '';
            if (this.customName) {
                labelText = this.customName;
            }
            if (this.$label && this.$label.innerText) {
                labelText = this.$label.innerText;
            }
            return labelText;
        };
    }
    validate(formValue) {
        this.validator = this.getCustomValidators(formValue.value);
        this.runValidation();
        return this.validator;
    }
    runValidation() {
    }
}
VonFormValidationBase.propDecorators = {
    ngModel: [{ type: Input }],
    name: [{ type: Input }],
    customName: [{ type: Input }],
    required: [{ type: Input }],
    requiredMessage: [{ type: Input }],
    equalTo: [{ type: Input }],
    equalIgnoreCase: [{ type: Input }],
    equalToMessage: [{ type: Input }]
};

const VALIDATION_MESSAGES = {
    ngModelRequired: 'You need to add [(ngModel)] into the element',
    EN: {
        requiredMessage: 'The field \'${name}\' is required',
        equalToMessage: 'The field \'${name}\' is not equal'
    },
    ES: {
        requiredMessage: 'El campo \'${name}\' es requerido',
        equalToMessage: 'El campo \'${name}\' no es igual'
    },
};

class VonFormValidationDirective extends VonFormValidationBase {
    constructor(element, renderer) {
        super(element);
        this.element = element;
        this.renderer = renderer;
        this.wrapperEl = {};
        this.messageEl = {};
        this.executeValidationEvent = () => {
            this.validator = this.getCustomValidators(this.ngModel);
            this.renderer.removeChild(this.wrapperEl, this.messageEl);
            const labelText = this.getLabelText();
            let message = '';
            let valid = false;
            if (this.validator.isEmpty) {
                message = this.requiredMessage || this.messages.requiredMessage;
            }
            else if (this.validator.isNotEqual) {
                message = this.equalToMessage || this.messages.equalToMessage;
            }
            else {
                valid = true;
            }
            if (!valid) {
                this.renderer.addClass(this.element.nativeElement, 'field__error');
                message = message.replace('${name}', labelText);
                this.messageEl.innerText = message;
                this.renderer.appendChild(this.wrapperEl, this.messageEl);
            }
            else {
                this.renderer.removeClass(this.element.nativeElement, 'field__error');
            }
            this.element.nativeElement.setAttribute('validation', valid);
        };
        this.getCustomValidators = (formValue) => {
            return Object.assign(Object.assign({}, this.validateRequired(formValue)), this.validateEqualTo(formValue));
        };
        this.validateRequired = (value) => {
            let validation = {};
            if (!this.required) {
                return {};
            }
            if (this.required && (value == null || value === '')) {
                validation = { isEmpty: true };
            }
            return validation;
        };
        this.validateEqualTo = (value) => {
            if (!this.equalTo) {
                return {};
            }
            let validation = {};
            const isNotEqual = this.equalIgnoreCase
                ? `${value}`.toLowerCase() !== `${this.equalTo}`.toLowerCase()
                : value !== this.equalTo;
            if (isNotEqual) {
                validation = { isNotEqual: true };
            }
            return validation;
        };
    }
    ngOnInit() {
        const $closestFormEl = this.element.nativeElement.form || this.getClosestForm(this.element.nativeElement);
        const isValidationEn = !$closestFormEl.hasAttribute('validation-es');
        this.messages = isValidationEn
            ? VALIDATION_MESSAGES.EN
            : VALIDATION_MESSAGES.ES;
        this.$label = $closestFormEl.querySelector(`label[for='${this.name}']`);
        if (this.required && this.$label) {
            this.$label.classList.add('field__label--required');
        }
        this.element.nativeElement.setAttribute('validation', true);
        this.wrapperEl = this.renderer.createElement('div');
        this.renderer.addClass(this.wrapperEl, 'form__field');
        this.renderer.insertBefore(this.element.nativeElement.parentElement, this.wrapperEl, this.element.nativeElement);
        this.renderer.appendChild(this.wrapperEl, this.element.nativeElement);
        if (this.$label) {
            this.renderer.addClass(this.$label, 'field__label');
            this.renderer.insertBefore(this.wrapperEl, this.$label, this.element.nativeElement);
        }
        this.messageEl = this.renderer.createElement('span');
        this.renderer.addClass(this.messageEl, 'field__error-message');
    }
    runValidation() {
        if (this.validator.isEmpty
            || this.validator.isNotEqual) {
            return;
        }
        this.renderer.removeClass(this.element.nativeElement, 'field__error');
    }
}
VonFormValidationDirective.decorators = [
    { type: Directive, args: [{
                selector: '[validation]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: VonFormValidationDirective, multi: true }
                ]
            },] }
];
VonFormValidationDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
VonFormValidationDirective.propDecorators = {
    executeValidationEvent: [{ type: HostListener, args: ['executeValidation',] }]
};

class VonFormModule {
}
VonFormModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    VonFormValidateDirective,
                    VonFormValidationDirective
                ],
                imports: [
                    FormsModule
                ],
                exports: [
                    FormsModule,
                    VonFormValidateDirective,
                    VonFormValidationDirective
                ],
                providers: []
            },] }
];

/*
 * Public API Surface of angular-form-validation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { VALIDATION_MESSAGES, VonFormModule, VonFormValidateDirective, VonFormValidationBase, VonFormValidationDirective };
//# sourceMappingURL=von-development-studio-angular-form-validation.js.map
